"use strict";(self.webpackChunklearninghouse=self.webpackChunklearninghouse||[]).push([[592],{9984:(c,u,r)=>{r.d(u,{H:()=>e,d:()=>t});var t=(()=>((t=t||{}).NUMERICAL="numerical",t.CATEGORICAL="categorical",t))(),e=(()=>((e=e||{}).CLASSIFIER="classifier",e.REGRESSOR="regressor",e))()},5559:(c,u,r)=>{r.d(u,{I:()=>a});var t=r(5619),e=r(2096);class a{constructor(){this.state$=new t.X(null)}handleSuccess(){return this.state$.next("success"),(0,e.of)(!0)}handleError(o){return this.state$.next(o.key),(0,e.of)(!1)}get isSuccess(){return"success"===this.state$.getValue()}}},4419:(c,u,r)=>{r.d(u,{a:()=>E});var t=r(8180),e=r(7398),a=r(5879),l=r(9637),o=r(1896);let E=(()=>{class s{constructor(n,_){this.authService=n,this.router=_}checkMinimumRole(n){return this.authService.role$.pipe((0,t.q)(1),(0,e.U)(_=>_?!!_.isMinimumRole(n)||this.router.createUrlTree(["/brains"]):this.router.createUrlTree(["/auth"])))}}return s.\u0275fac=function(n){return new(n||s)(a.LFG(l.e),a.LFG(o.F0))},s.\u0275prov=a.Yz7({token:s,factory:s.\u0275fac,providedIn:"root"}),s})()}}]);